#ifndef __COMMON_H
#define __COMMON_H

#define DIGIT_MAX 8

#define DISABLE 0
#define ENABLE  1

#define DUTY_CYCLE    50
#define DUTY_MAX      32
#define DUTY_MIN      12
#define DUTY_PWM_AMP1 12
#define DUTY_PWM_AMP2 134

#define TIME_MODE       0
#define DATE_MODE       1
#define YEAR_MODE       2
#define TEMP_MODE       3
#define PRESS_MODE      4
#define BRIGHTNESS      5
#define LIGHT_SENSOR_EN 6
#define LIGHT_SENSOR_TH 7
#define LIGHT_SENSOR_SW 8
#define GPS_EN          9
#define GPS_TZ          10

#define CONFIG_NONE            0
#define CONFIG_MIN             1
#define CONFIG_HOUR            2
#define CONFIG_DAY             3
#define CONFIG_DATE            4
#define CONFIG_MONTH           5
#define CONFIG_YEAR            6
#define CONFIG_CENTURY         7
#define CONFIG_BRIGHTNESS      8
#define CONFIG_LIGHT_SENSOR_EN 9
#define CONFIG_LIGHT_SENSOR_TH 10
#define CONFIG_LIGHT_SENSOR_SW 11
#define CONFIG_GPS_EN          12
#define CONFIG_GPS_TZ          13

#define MINUS       36
#define SUPRESS     37
#define ALL_ON      38
#define FONT_OFFSET 55

#define TEMP_TURNON 20

#define COLON_PWM 0
#define COLON_SW  1
#define COLON_ON  2
#define COLON_OFF 3

#define NIXIE_DRIVER PORTA

#define HVPS_SW_PORT PORTE
#define HVPS_SW      PE2

#define SEG_HIGH PORTC
#define SEG_LOW  PORTF

#define PWM_MAX 250
#define PWM_LOW PWM_MIN + 5
#define PWM_MIN 10
#define PWM_OFF 0

#define POWER_LED_PORT      PORTG
#define POWER_LED           PG3
#define ERROR_LED_PORT      PORTG
#define ERROR_LED           PG4
#define GPS_STATUS_LED_PORT PORTE
#define GPS_STATUS_LED      PE3
#define GPS_STATUS_SEG_PORT PORTB
#define GPS_STATUS_SEG      PB5

#define SEG_DOT_PWM OCR1A
#define L_COLON_PWM OCR1B
#define R_COLON_PWM OCR1C
#define COLON_PORT  DDRB
#define L_COLON     PB6
#define R_COLON     PB7

#define GPS_PS_PORT PORTG
#define GPS_PS      PG2

#define SW_2_PIN PIND
#define SW_2     PD4
#define SW_3_PIN PIND
#define SW_3     PD5
#define SW_4_PIN PIND
#define SW_4     PD6
#define SW_5_PIN PIND
#define SW_5     PD7

#define ENC_1_A_PIN PINE
#define ENC_1_A     PE6
#define ENC_1_B_PIN PING
#define ENC_1_B     PG1
#define ENC_2_A_PIN PINE
#define ENC_2_A     PE7
#define ENC_2_B_PIN PING
#define ENC_2_B     PG0

#define TEST_1 1
#define TEST_2 2

void wait_ms(uint16_t t);

void on_hvps(void);
void off_hvps(void);

void set_segment(uint8_t msb, uint8_t lsb);
void set_dot(uint8_t pwm);

void on_power_led(void);
void off_power_led(void);
void toggle_power_led(void);
void on_error_led(void);
void off_error_led(void);
void toggle_error_led(void);
void on_gps_status_led(void);
void off_gps_status_led(void);
void toggle_gps_status_led(void);
void on_gps_status_seg(void);
void off_gps_status_seg(void);

void start_gps(void);
void stop_gps(void);
void toggle_gps(uint8_t val);

void max_brightness(void);
void normal_brightness(void);
void limit_brightness(void);
void set_brightness(uint8_t num, uint8_t val);
void set_source_brightness(uint8_t val);
void set_colon_brightness(uint16_t val);
void set_colon_source_brightness(uint16_t val);
void off_all_TUBES(void);
void input_colon(void);
void output_colon(void);

uint8_t get_day_char1(uint8_t val);
uint8_t get_day_char2(uint8_t val);

uint8_t get_max_date(void);

#endif
