#include <avr/io.h>

#include "ad.h"
#include "common.h"
#include "eeprom.h"
#include "extern.h"
#include "rtc.h"

// read eeprom setting
void read_setting(void){
	uint8_t i;

	// mode
	eeprom_busy_wait();
	mode = eeprom_read_byte((uint8_t *)E_MODE);
	if(mode < 0 || mode > GPS_TZ){
		mode = E_MODE_DEFAULT;
		eeprom_busy_wait();
		eeprom_update_byte((uint8_t *)E_MODE, E_MODE_DEFAULT);
	}

	// colon mode
	eeprom_busy_wait();
	colon_mode = eeprom_read_byte((uint8_t *)E_COLON_MODE);
	if(colon_mode < 0 || colon_mode > COLON_OFF){
		colon_mode = E_COLON_MODE_DEFAULT;
		eeprom_busy_wait();
		eeprom_update_byte((uint8_t *)E_COLON_MODE, E_COLON_MODE_DEFAULT);
	}

	// century
	eeprom_busy_wait();
	century = eeprom_read_byte((uint8_t *)E_CENTURY);
	if(century < 0 || century > 99){
		century = E_CENTURY_DEFAULT;
		eeprom_busy_wait();
		eeprom_update_byte((uint8_t *)E_CENTURY, E_CENTURY_DEFAULT);
	}

	// duty
	eeprom_busy_wait();
	duty = eeprom_read_byte((uint8_t *)E_DUTY);
	if(duty < DUTY_MIN || duty > DUTY_MAX){
		duty = E_DUTY_DEFAULT;
		eeprom_busy_wait();
		eeprom_update_byte((uint8_t *)E_DUTY, E_DUTY_DEFAULT);
	}
	for(i=0; i<DIGIT_MAX; i++){
		digit_brightness[i] = duty;
	}
	for(i=0; i<DIGIT_MAX; i++){
		digit_main[i] = SUPRESS;
	}

	// colon brightness
	eeprom_busy_wait();
	colon_brightness[1] = eeprom_read_byte((uint8_t *)E_COLON_BRIGHTNESS);
	if(colon_brightness[1] < PWM_MIN || colon_brightness[1] > PWM_MAX){
		colon_brightness[1] = E_COLON_BRIGHTNESS_DEFAULT;
		eeprom_busy_wait();
		eeprom_update_byte((uint8_t *)E_COLON_BRIGHTNESS, E_COLON_BRIGHTNESS_DEFAULT);
	}
	colon_brightness[1] = PWM_MAX;
	colon_brightness[0] = colon_brightness[1];

	// use gps
	eeprom_busy_wait();
	gps_en = eeprom_read_byte((uint8_t *)E_GPS_EN);
	if(gps_en < 0 || gps_en > 1){
		gps_en = E_GPS_EN_DEFAULT;
		eeprom_busy_wait();
		eeprom_update_byte((uint8_t *)E_GPS_EN, E_GPS_EN_DEFAULT);
	}

	// gps timezone
	eeprom_busy_wait();
	gps_timezone = eeprom_read_byte((uint8_t *)E_GPS_TIMEZONE);
	if(gps_timezone < GPS_TIMEZONE_MIN || gps_timezone > GPS_TIMEZONE_MAX){
		gps_timezone = E_GPS_TIMEZONE_DEFAULT;
		eeprom_busy_wait();
		eeprom_update_byte((uint8_t *)E_GPS_TIMEZONE, E_GPS_TIMEZONE_DEFAULT);
	}

	// use light sensor
	eeprom_busy_wait();
	light_sensor_en = eeprom_read_byte((uint8_t *)E_LIGHT_SENSOR_EN);
	if(light_sensor_en < 0 || light_sensor_en > 1){
		light_sensor_en = E_LIGHT_SENSOR_EN_DEFAULT;
		eeprom_busy_wait();
		eeprom_update_byte((uint8_t *)E_LIGHT_SENSOR_EN, E_LIGHT_SENSOR_EN_DEFAULT);
	}

	// light sensor threshold
	eeprom_busy_wait();
	light_sensor_th = eeprom_read_word((uint16_t *)E_LIGHT_SENSOR_TH);
	if(light_sensor_th < LIGHT_SENSOR_TH_MIN || light_sensor_th > LIGHT_SENSOR_TH_MAX){
		light_sensor_th = E_LIGHT_SENSOR_TH_DEFAULT;
		eeprom_busy_wait();
		eeprom_update_word((uint16_t *)E_LIGHT_SENSOR_TH, E_LIGHT_SENSOR_TH_DEFAULT);
	}

	// light sensor sw
	eeprom_busy_wait();
	light_sensor_sw = eeprom_read_byte((uint8_t *)E_LIGHT_SENSOR_SW);
	if(light_sensor_sw < 0 || light_sensor_sw > 1){
		light_sensor_sw = E_LIGHT_SENSOR_SW_DEFAULT;
		eeprom_busy_wait();
		eeprom_update_byte((uint8_t *)E_LIGHT_SENSOR_SW, E_LIGHT_SENSOR_SW_DEFAULT);
	}
}

// reset eeprom setting
void reset_setting(void){
	uint8_t i;

	rtc_write(RTC_SEC_ADDR, RTC_SEC_DEFAULT);
	rtc_write(RTC_MIN_ADDR, RTC_MIN_DEFAULT);
	rtc_write(RTC_HOUR_ADDR, RTC_HOUR_DEFAULT);
	rtc_write(RTC_DAY_ADDR, RTC_DAY_DEFAULT);
	rtc_write(RTC_DATE_ADDR, RTC_DATE_DEFAULT);
	rtc_write(RTC_MONTH_ADDR, RTC_MONTH_DEFAULT);
	rtc_write(RTC_YEAR_ADDR, RTC_YEAR_DEFAULT);

	rtc_write(RTC_ALARM1_SEC_ADDR, RTC_ALARM1_SEC_DEFAULT);
	rtc_write(RTC_ALARM1_MIN_ADDR, RTC_ALARM1_MIN_DEFAULT);
	rtc_write(RTC_ALARM1_HOUR_ADDR, RTC_ALARM1_HOUR_DEFAULT);
	rtc_write(RTC_ALARM1_DATE_ADDR, RTC_ALARM1_DATE_DEFAULT);

	rtc_write(RTC_ALARM2_MIN_ADDR, RTC_ALARM2_MIN_DEFAULT);
	rtc_write(RTC_ALARM2_HOUR_ADDR, RTC_ALARM2_HOUR_DEFAULT);
	rtc_write(RTC_ALARM2_DATE_ADDR, RTC_ALARM2_DATE_DEFAULT);

	// mode
	mode = E_MODE_DEFAULT;
	eeprom_busy_wait();
	eeprom_update_byte((uint8_t *)E_MODE, E_MODE_DEFAULT);

	// colon mode
	colon_mode = E_COLON_MODE_DEFAULT;
	eeprom_busy_wait();
	eeprom_update_byte((uint8_t *)E_COLON_MODE, E_COLON_MODE_DEFAULT);

	// century
	century = E_CENTURY_DEFAULT;
	eeprom_busy_wait();
	eeprom_update_byte((uint8_t *)E_CENTURY, E_CENTURY_DEFAULT);

	// duty
	duty = E_DUTY_DEFAULT;
	eeprom_busy_wait();
	eeprom_update_byte((uint8_t *)E_DUTY, E_DUTY_DEFAULT);
	for(i=0; i<DIGIT_MAX; i++){
		digit_brightness[i] = duty;
	}
	for(i=0; i<DIGIT_MAX; i++){
		digit_main[i] = SUPRESS;
	}

	// colon brightness
	colon_brightness[1] = E_COLON_BRIGHTNESS_DEFAULT;
	eeprom_busy_wait();
	eeprom_update_byte((uint8_t *)E_COLON_BRIGHTNESS, E_COLON_BRIGHTNESS_DEFAULT);
	colon_brightness[0] = colon_brightness[1];

	// use gps
	gps_en = E_GPS_EN_DEFAULT;
	eeprom_busy_wait();
	eeprom_update_byte((uint8_t *)E_GPS_EN, E_GPS_EN_DEFAULT);

	// gps timezone
	gps_timezone = E_GPS_TIMEZONE_DEFAULT;
	eeprom_busy_wait();
	eeprom_update_byte((uint8_t *)E_GPS_TIMEZONE, E_GPS_TIMEZONE_DEFAULT);

	// use light sensor
	light_sensor_en = E_LIGHT_SENSOR_EN_DEFAULT;
	eeprom_busy_wait();
	eeprom_update_byte((uint8_t *)E_LIGHT_SENSOR_EN, E_LIGHT_SENSOR_EN_DEFAULT);

	// light sensor threshold
	light_sensor_th = E_LIGHT_SENSOR_TH_DEFAULT;
	eeprom_busy_wait();
	eeprom_update_word((uint16_t *)E_LIGHT_SENSOR_TH, E_LIGHT_SENSOR_TH_DEFAULT);

	// light sensor sw
	light_sensor_sw = E_LIGHT_SENSOR_SW_DEFAULT;
	eeprom_busy_wait();
	eeprom_update_byte((uint8_t *)E_LIGHT_SENSOR_SW, E_LIGHT_SENSOR_SW_DEFAULT);

	on_error_led();
	off_hvps();

	while(1); // loop
}
