#include <avr/io.h>
#include <avr/interrupt.h>

#include "ad.h"
#include "common.h"
#include "extern.h"
#include "i2c.h"
#include "int.h"
#include "rtc.h"

void init_int(void){
	// INT4 RTC interrupt
	// INT5 for GPS 1pps
	// INT6 for ENC1 A
	// INT7 for ENC2 A
	// encoders are checked by TIMER3
	// EIMSK |= _BV(INT4) | _BV(INT5) | _BV(INT6) | _BV(INT7);
	EIMSK |= _BV(INT4) | _BV(INT5);
	// INT4 RTC:  clock up/down edge
	// INT5 GPS:  clock up edge
	// INT6 ENC1: clock up edge
	// INT7 ENC2: clock up edge
	//EICRB = _BV(ISC40) | _BV(ISC41) | _BV(ISC50) | _BV(ISC51) | _BV(ISC60) | _BV(ISC61) | _BV(ISC70);
	EICRB = _BV(ISC40) | _BV(ISC50) | _BV(ISC51);
}

// RTC 1Hz clock
ISR(INT4_vect){
	static uint8_t  test_num = 0;

	// get time and date from RTC
	// this data is referenced from many funcs and area
	// read everytime
	rtc_sec = rtc_read(RTC_SEC_ADDR);
	rtc_min = rtc_read(RTC_MIN_ADDR);
	rtc_hour = rtc_read(RTC_HOUR_ADDR);

	// check AMPM or 24Hr
	if(rtc_hour & AMPM_FLAG){
		digit_time[0] = (rtc_hour & 0b00010000) >> 4;
	}else{
		digit_time[0] = (rtc_hour & 0b00110000) >> 4;
	}

	// set time digit
	digit_time[1] = rtc_hour & 0b00001111;
	digit_time[2] = SUPRESS;
	digit_time[3] = rtc_min >> 4;
	digit_time[4] = rtc_min & 0b00001111;
	digit_time[5] = SUPRESS;
	digit_time[6] = rtc_sec >> 4;
	digit_time[7] = rtc_sec & 0b00001111;

	// get date from RTC
	rtc_day = rtc_read(RTC_DAY_ADDR);
	rtc_date = rtc_read(RTC_DATE_ADDR);
	rtc_month = rtc_read(RTC_MONTH_ADDR);
	rtc_year = rtc_read(RTC_YEAR_ADDR);

	// set date digit
	digit_date[0] = rtc_year >> 4;
	digit_date[1] = rtc_year & 0b00001111;
	digit_date[2] = SUPRESS;
	digit_date[3] = (rtc_month & 0b00010000) >> 4;
	digit_date[4] = rtc_month & 0b00001111;
	digit_date[5] = SUPRESS;
	digit_date[6] = rtc_date >> 4;
	digit_date[7] = rtc_date & 0b00001111;

	// MODE
	switch(mode){
	case TIME_MODE:
		if(config_mode == CONFIG_NONE){
			if(test_mode == DISABLE){ // normal
				digit_main[0] = digit_time[0];
				digit_main[1] = digit_time[1];
				digit_main[2] = digit_time[2];
				digit_main[3] = digit_time[3];
				digit_main[4] = digit_time[4];
				digit_main[5] = digit_time[5];
				digit_main[6] = digit_time[6];
				digit_main[7] = digit_time[7];

				// set AM/PM indicate dot
				// AMPM mode and time is AM
				if((rtc_hour & AMPM_FLAG) && !(rtc_hour & PM_FLAG)){
					digit_dot[0] = PWM_MAX;
				}else{
					digit_dot[0] = PWM_OFF;
				}
				// AMPM mode and time is PM
				if((rtc_hour & AMPM_FLAG) && (rtc_hour & PM_FLAG)){
					digit_dot[1] = PWM_MAX;
				}else{
					digit_dot[1] = PWM_OFF;
				}
				digit_dot[2] = PWM_OFF;
				digit_dot[3] = PWM_OFF;
				digit_dot[4] = PWM_OFF;
				digit_dot[5] = PWM_OFF;
				digit_dot[6] = PWM_OFF;
				//digit_dot[7] = PWM_OFF;
				if(gps_en == ENABLE && gps_count < GPS_COUNT){
					on_gps_status_seg();
				}else{
					off_gps_status_seg();
				}
			}else if(test_mode == TEST_1){ // character test
				digit_main[0] = test_num;
				digit_main[1] = test_num;
				digit_main[2] = test_num;
				digit_main[3] = test_num;
				digit_main[4] = test_num;
				digit_main[5] = test_num;
				digit_main[6] = test_num;
				digit_main[7] = test_num;
				digit_dot[0] = PWM_MAX;
				digit_dot[1] = PWM_MAX;
				digit_dot[2] = PWM_MAX;
				digit_dot[3] = PWM_MAX;
				digit_dot[4] = PWM_MAX;
				digit_dot[5] = PWM_MAX;
				digit_dot[6] = PWM_MAX;
				digit_dot[7] = PWM_MAX;
				test_num++;
				if(test_num >= ALL_ON){test_num = 0;}
			}else if(test_mode == TEST_2){ // tube test
				digit_main[0] = ALL_ON;
				digit_main[1] = ALL_ON;
				digit_main[2] = ALL_ON;
				digit_main[3] = ALL_ON;
				digit_main[4] = ALL_ON;
				digit_main[5] = ALL_ON;
				digit_main[6] = ALL_ON;
				digit_main[7] = ALL_ON;
				digit_dot[0] = PWM_MAX;
				digit_dot[1] = PWM_MAX;
				digit_dot[2] = PWM_MAX;
				digit_dot[3] = PWM_MAX;
				digit_dot[4] = PWM_MAX;
				digit_dot[5] = PWM_MAX;
				digit_dot[6] = PWM_MAX;
				digit_dot[7] = PWM_MAX;
			}
		}
		break;

	case DATE_MODE:
		if(config_mode == CONFIG_NONE){
			digit_main[0] = digit_date[3];
			digit_main[1] = digit_date[4];
			digit_main[2] = SUPRESS;
			digit_main[3] = digit_date[6];
			digit_main[4] = digit_date[7];
			digit_main[5] = SUPRESS;
			digit_main[6] = get_day_char1(rtc_day);
			digit_main[7] = get_day_char2(rtc_day);
			digit_dot[0] = PWM_OFF;
			digit_dot[1] = PWM_OFF;
			digit_dot[2] = PWM_OFF;
			digit_dot[3] = PWM_OFF;
			digit_dot[4] = PWM_OFF;
			digit_dot[5] = PWM_OFF;
			digit_dot[6] = PWM_OFF;
			digit_dot[7] = PWM_OFF;
		}
		break;

	case YEAR_MODE:
		if(config_mode == CONFIG_NONE){
			digit_main[0] = century / 10;
			digit_main[1] = century - digit_main[0] * 10;
			digit_main[2] = SUPRESS;
			digit_main[3] = digit_date[0];
			digit_main[4] = digit_date[1];
			digit_main[5] = SUPRESS;
			digit_main[6] = SUPRESS;
			digit_main[7] = SUPRESS;
			digit_dot[0] = PWM_OFF;
			digit_dot[1] = PWM_OFF;
			digit_dot[2] = PWM_OFF;
			digit_dot[3] = PWM_OFF;
			digit_dot[4] = PWM_OFF;
			digit_dot[5] = PWM_OFF;
			digit_dot[6] = PWM_OFF;
			digit_dot[7] = PWM_OFF;
		}
		break;

	case TEMP_MODE:
		MS5607_get();
		digit_main[0] = SUPRESS;
		digit_main[1] = SUPRESS;
		digit_main[2] = SUPRESS;
		digit_main[3] = digit_temparature[4];
		digit_main[4] = digit_temparature[5];
		digit_main[5] = SUPRESS;
		digit_main[6] = SUPRESS;
		digit_main[7] = digit_temparature[6];
		digit_dot[0] = PWM_OFF;
		digit_dot[1] = PWM_OFF;
		digit_dot[2] = PWM_OFF;
		digit_dot[3] = PWM_OFF;
		digit_dot[4] = PWM_OFF;
		digit_dot[5] = PWM_OFF;
		digit_dot[6] = PWM_MAX;
		digit_dot[7] = PWM_OFF;
		break;

	case PRESS_MODE:
		MS5607_get();
		digit_main[0] = digit_pressure[2];
		digit_main[1] = digit_pressure[3];
		digit_main[2] = SUPRESS;
		digit_main[3] = digit_pressure[4];
		digit_main[4] = digit_pressure[5];
		digit_main[5] = SUPRESS;
		digit_main[6] = SUPRESS;
		digit_main[7] = digit_pressure[6];
		digit_dot[0] = PWM_OFF;
		digit_dot[1] = PWM_OFF;
		digit_dot[2] = PWM_OFF;
		digit_dot[3] = PWM_OFF;
		digit_dot[4] = PWM_OFF;
		digit_dot[5] = PWM_OFF;
		digit_dot[6] = PWM_MAX;
		digit_dot[7] = PWM_OFF;
		break;

	case BRIGHTNESS:
		if(config_mode == CONFIG_NONE){
			normal_brightness();
			digit_main[0] = 'B' - FONT_OFFSET;
			digit_main[1] = 'R' - FONT_OFFSET;
			digit_main[2] = SUPRESS;
			digit_main[3] = 'I' - FONT_OFFSET;
			digit_main[4] = SUPRESS;
			digit_main[5] = SUPRESS;
			digit_main[6] = (duty - DUTY_MIN) / 10;
			digit_main[7] = (duty - DUTY_MIN) - digit_main[6] * 10;
			digit_dot[0] = PWM_OFF;
			digit_dot[1] = PWM_OFF;
			digit_dot[2] = PWM_OFF;
			digit_dot[3] = PWM_OFF;
			digit_dot[4] = PWM_OFF;
			digit_dot[5] = PWM_OFF;
			digit_dot[6] = PWM_OFF;
			digit_dot[7] = PWM_OFF;
		}
		break;

	case LIGHT_SENSOR_EN:
		if(config_mode == CONFIG_NONE){
			normal_brightness();
			digit_main[0] = 'S' - FONT_OFFSET;
			digit_main[1] = 'E' - FONT_OFFSET;
			digit_main[2] = SUPRESS;
			digit_main[3] = 'N' - FONT_OFFSET;
			if(light_sensor_en == ENABLE){
				digit_main[4] = SUPRESS;
				digit_main[5] = SUPRESS;
				digit_main[6] = 'O' - FONT_OFFSET;
				digit_main[7] = 'N' - FONT_OFFSET;
			}else{
				digit_main[4] = 'O' - FONT_OFFSET;
				digit_main[5] = SUPRESS;
				digit_main[6] = 'F' - FONT_OFFSET;
				digit_main[7] = 'F' - FONT_OFFSET;
			}
			digit_dot[0] = PWM_OFF;
			digit_dot[1] = PWM_OFF;
			digit_dot[2] = PWM_OFF;
			digit_dot[3] = PWM_OFF;
			digit_dot[4] = PWM_OFF;
			digit_dot[5] = PWM_OFF;
			digit_dot[6] = PWM_OFF;
			digit_dot[7] = PWM_OFF;
		}
		break;

	case LIGHT_SENSOR_TH:
		if(config_mode == CONFIG_NONE){
			normal_brightness();
			digit_main[0] = 'T' - FONT_OFFSET;
			digit_main[1] = 'H' - FONT_OFFSET;
			digit_main[2] = SUPRESS;
			digit_main[3] = 'R' - FONT_OFFSET;
			digit_main[4] = light_sensor_th / 100;
			digit_main[5] = SUPRESS;
			digit_main[6] = (light_sensor_th - digit_main[4] * 100) / 10;
			digit_main[7] = light_sensor_th - digit_main[4] * 100 - digit_main[6] * 10;
			digit_dot[0] = PWM_OFF;
			digit_dot[1] = PWM_OFF;
			digit_dot[2] = PWM_OFF;
			digit_dot[3] = PWM_OFF;
			digit_dot[4] = PWM_OFF;
			digit_dot[5] = PWM_OFF;
			digit_dot[6] = PWM_OFF;
			digit_dot[7] = PWM_OFF;
		}
		break;

	case LIGHT_SENSOR_SW:
		if(config_mode == CONFIG_NONE){
			digit_main[0] = 'E' - FONT_OFFSET;
			digit_main[1] = 'C' - FONT_OFFSET;
			digit_main[2] = SUPRESS;
			digit_main[3] = 'O' - FONT_OFFSET;
			if(light_sensor_sw == ENABLE){
				digit_main[4] = SUPRESS;
				digit_main[5] = SUPRESS;
				digit_main[6] = 'O' - FONT_OFFSET;
				digit_main[7] = 'N' - FONT_OFFSET;
			}else{
				digit_main[4] = 'O' - FONT_OFFSET;
				digit_main[5] = SUPRESS;
				digit_main[6] = 'F' - FONT_OFFSET;
				digit_main[7] = 'F' - FONT_OFFSET;
			}
			digit_dot[0] = PWM_OFF;
			digit_dot[1] = PWM_OFF;
			digit_dot[2] = PWM_OFF;
			digit_dot[3] = PWM_OFF;
			digit_dot[4] = PWM_OFF;
			digit_dot[5] = PWM_OFF;
			digit_dot[6] = PWM_OFF;
			digit_dot[7] = PWM_OFF;
		}
		break;

	case GPS_EN:
		if(config_mode == CONFIG_NONE){
			normal_brightness();
			digit_main[0] = 'G' - FONT_OFFSET;
			digit_main[1] = 'P' - FONT_OFFSET;
			digit_main[2] = SUPRESS;
			digit_main[3] = 'S' - FONT_OFFSET;
			if(gps_en == ENABLE){
				digit_main[4] = SUPRESS;
				digit_main[5] = SUPRESS;
				digit_main[6] = 'O' - FONT_OFFSET;
				digit_main[7] = 'N' - FONT_OFFSET;
			}else{
				digit_main[4] = 'O' - FONT_OFFSET;
				digit_main[5] = SUPRESS;
				digit_main[6] = 'F' - FONT_OFFSET;
				digit_main[7] = 'F' - FONT_OFFSET;
			}
			digit_dot[0] = PWM_OFF;
			digit_dot[1] = PWM_OFF;
			digit_dot[2] = PWM_OFF;
			digit_dot[3] = PWM_OFF;
			digit_dot[4] = PWM_OFF;
			digit_dot[5] = PWM_OFF;
			digit_dot[6] = PWM_OFF;
			digit_dot[7] = PWM_OFF;
		}
		break;

	case GPS_TZ:
		if(config_mode == CONFIG_NONE){
			digit_main[0] = 'T' - FONT_OFFSET;
			digit_main[1] = 'Z' - FONT_OFFSET;
			digit_main[2] = SUPRESS;
			digit_main[3] = SUPRESS;
			if(gps_timezone >= 0){
				digit_main[4] = SUPRESS;
				digit_main[5] = SUPRESS;
				digit_main[6] = gps_timezone / 10;
				digit_main[7] = gps_timezone - digit_main[6] * 10;
			}else{
				digit_main[4] = MINUS;
				digit_main[5] = SUPRESS;
				digit_main[6] = (~(gps_timezone) + 1) / 10;
				digit_main[7] = (~(gps_timezone) + 1) - digit_main[6] * 10;
			}
			digit_dot[0] = PWM_OFF;
			digit_dot[1] = PWM_OFF;
			digit_dot[2] = PWM_OFF;
			digit_dot[3] = PWM_OFF;
			digit_dot[4] = PWM_OFF;
			digit_dot[5] = PWM_OFF;
			digit_dot[6] = PWM_OFF;
			digit_dot[7] = PWM_OFF;
		}
		break;
	
	default:
		mode = 0;
		break;
	}

	// light sensor
	if(light_sensor_en == ENABLE){
		get_light_sensor_val();
		if(config_mode == CONFIG_NONE && (mode == TIME_MODE || mode == DATE_MODE || mode == YEAR_MODE || mode == TEMP_MODE || mode == PRESS_MODE)){
			if(light_sensor <= light_sensor_th && temp_turnon == 0){
				if(temp_turnon_flag == 1){temp_turnon = TEMP_TURNON;}
				temp_turnon_flag = 0;
				if(light_sensor_sw == 0){
					limit_brightness();
				}else{
					off_all_TUBES();
				}
			}else{
				temp_turnon_flag = 0;
				normal_brightness();
			}
		}
	}

	// gps count
	if(gps_count < GPS_COUNT * 2){
		gps_count++;
	}
	off_gps_status_led();

	// temp_turnon
	if(temp_turnon > 0){temp_turnon--;}
}

// GPS 1pps
// sync RTC by GPS time
ISR(INT5_vect){
	uint8_t temp_hour;
	uint8_t hour_flag = 0;

	// update by 10sec @ sec == 5
	if(gpstime.valid == 1 && gpstime.time[5] == '4' &&  gps_en == ENABLE){
		gpstime.valid = 0;

		// adjust 1pps timing(add 1sec)
		// second must be 0 - 8, 9 is not good
		// because following char to uint method
		gpstime.time[5]++;

		// char to uint
		gpstime.time[0] -= 48;
		gpstime.time[1] -= 48;
		gpstime.time[2] -= 48;
		gpstime.time[3] -= 48;
		gpstime.time[4] -= 48;
		gpstime.time[5] -= 48;

		// calc UTC hour & adjust timezone
		temp_hour = gpstime.time[0] * 10 + gpstime.time[1];
		temp_hour += gps_timezone;

		// check 24hr ouverflow
		if(temp_hour >= 24){
			temp_hour -= 24;
		}

		// check RTC AM/PM or 24hr
		if((rtc_hour & AMPM_FLAG) && (temp_hour >= 12)){
			if(temp_hour != 12){temp_hour -= 12;}
			hour_flag = (AMPM_FLAG | PM_FLAG);
		}else if((rtc_hour & AMPM_FLAG) && (temp_hour < 12)){
			if(temp_hour == 0){temp_hour = 12;}
			hour_flag = AMPM_FLAG;
		}

		// to BCD
		gpstime.time[0] = temp_hour / 10;
		gpstime.time[1] = (temp_hour - gpstime.time[0] * 10);

		// write RTC
		rtc_write(RTC_SEC_ADDR,  (gpstime.time[4] << 4) | gpstime.time[5]);
		rtc_write(RTC_MIN_ADDR,  (gpstime.time[2] << 4) | gpstime.time[3]);
		rtc_write(RTC_HOUR_ADDR, (gpstime.time[0] << 4) | gpstime.time[1] | hour_flag);

		// update flag
		gps_count = 0;

	}
	// on status
	on_gps_status_led();
}

/*
ISR(INT6_vect){
}

ISR(INT7_vect){
}
*/

