#include <avr/interrupt.h>
#include <avr/io.h>

#include "ad.h"
#include "common.h"
#include "eeprom.h"
#include "font.h"
#include "i2c.h"
#include "int.h"
#include "port.h"
#include "rtc.h"
#include "timer.h"
#include "uart.h"

volatile uint8_t  digit = 0;                                     // digit location 0-5

volatile uint8_t  mode = E_MODE_DEFAULT;                         // mode
volatile uint8_t  colon_mode = E_COLON_MODE_DEFAULT;             // colon pattern
volatile uint8_t  colon_brightness[2] = {0};                     // colon brightnesss
volatile uint8_t  config_mode = CONFIG_NONE;                     // config mode

volatile uint8_t  duty = E_DUTY_DEFAULT;                         // duty
volatile uint8_t  duty_count = 0;                                // duty count

volatile uint8_t  century = E_CENTURY_DEFAULT;                   // century(century is not stored in RTC)
volatile uint8_t  rtc_hour, rtc_min, rtc_sec;                    // RTC raw data
volatile uint8_t  rtc_year, rtc_month, rtc_date, rtc_day;        // RTC raw data

volatile uint8_t  digit_main[8] = {0};                           // digit 0-9 A-Z
volatile uint8_t  digit_brightness[8] = {0};                     // digit brightness
volatile uint8_t  digit_time[8] = {0};                           // digit for time
volatile uint8_t  digit_date[8] = {0};                           // digit for date
volatile uint8_t  digit_dot[8]  = {0};                           // digit for dot

volatile uint8_t  digit_pressure[8] = {0};
volatile uint8_t  digit_temparature[8] = {0};
volatile uint64_t temparature, pressure;

volatile uint8_t  gps_en = E_GPS_EN_DEFAULT;                     // use gps flag
volatile uint32_t gps_count = GPS_COUNT;                         // last update count
volatile GPSTIME  gpstime = {{0}};                               // record GPS time
volatile int8_t   gps_timezone = E_GPS_TIMEZONE_DEFAULT;         // GPS timezone

volatile uint8_t  light_sensor_en = E_LIGHT_SENSOR_EN_DEFAULT;   // use light sensor flag
volatile uint16_t light_sensor = 0;                              // light sensor adc value
volatile uint16_t light_sensor_th = E_LIGHT_SENSOR_TH_DEFAULT;   // light sensor threshold
volatile uint8_t  light_sensor_sw = E_LIGHT_SENSOR_SW;           // light sensor switch flag

volatile uint8_t  temp_turnon_flag = DISABLE;
volatile uint8_t  temp_turnon = 0;                               // make normal brightness temporarily

volatile uint8_t  test_mode = DISABLE;

int main(void){
	init_port();
	read_setting();
	init_timer();
	init_int();
	init_rtc();
	init_adc();
	init_uart();
	reset_MS5607();
	if(gps_en == ENABLE){start_gps();}
	on_hvps();

	// all reset
	if(bit_is_clear(SW_2_PIN, SW_2) && bit_is_clear(SW_4_PIN, SW_4)){
		rtc_clear();
		reset_setting();
	}

	// test mode 1
	if(bit_is_clear(SW_3_PIN, SW_3)){
		test_mode = TEST_1;
	}

	// test mode 2
	if(bit_is_clear(SW_5_PIN, SW_5)){
		test_mode = TEST_2;
	}

	sei();

	on_power_led();

	while(1);

	// WTF?
	on_error_led();
}
