#include <avr/io.h>
#include <avr/pgmspace.h>
#include <util/delay.h>

#include "eeprom.h"
#include "extern.h"
#include "rtc.h"
#include "xitoa.h"

// init RTC
void init_rtc(void){
	uint8_t status;

	SPCR = _BV(SPE) | _BV(MSTR) | _BV(CPHA); // SPI Master Mode 0
	SPSR = _BV(SPI2X);

	// wait until oscillator is started
	// when ocillator is started "RTC_CONTROL_EOSC" is 0
	do{
		status = rtc_read(RTC_CONTROL_ADDR);
	}while(status & RTC_CONTROL_EOSC);

	rtc_raw_write(RTC_CONTROL_ADDR, 0b00000000); // 1Hz, INT output
	rtc_raw_write(RTC_CONTROL_STATUS_ADDR, (RTC_STATUS_OSF | RTC_STATUS_BB32 | RTC_STATUS_EN32));
}

/*
void SPI_out(uint8_t data){
	SPDR = data;
	loop_while_bit_is_set(SPSR, SPIF);
}
*/

// check century flag in month address
void check_century(void){
	//uint8_t century;
	uint8_t buf[3];

	// read manulally, prevent loop
	if((rtc_month & CENTURY_FLAG) != 0){
		century++;
		if(century > 99){century = 0;}
		xsprintf((char *)buf, PSTR("%02d"), century);
		buf[0] -= 48;
		buf[1] -= 48;
		digit_date[0] = buf[0];
		digit_date[1] = buf[1];
		eeprom_busy_wait();
		eeprom_update_byte((uint8_t *)E_CENTURY, century);
		rtc_month &= ~CENTURY_FLAG;
		RTC_CS_PORT &= ~_BV(RTC_CS);
		SPDR = (RTC_MONTH_ADDR + 0x80);
		loop_until_bit_is_set(SPSR, SPIF);
		SPDR = rtc_month;
		loop_until_bit_is_set(SPSR, SPIF);
		RTC_CS_PORT |= _BV(RTC_CS);
		RTC_CS_PORT &= ~_BV(RTC_CS);
		RTC_CS_PORT |= _BV(RTC_CS);
	}
}

// read rtc
uint8_t rtc_read(uint8_t addr){
	uint8_t temp;

	if(addr == RTC_MONTH_ADDR){
		check_century();
	}
	RTC_CS_PORT &= ~_BV(RTC_CS);
	SPDR = addr;
	loop_until_bit_is_set(SPSR, SPIF);
	SPDR = addr;
	loop_until_bit_is_set(SPSR, SPIF);
	temp = SPDR;
	RTC_CS_PORT |= _BV(RTC_CS);

	return temp;
}

// write rtc
void rtc_write(uint8_t addr, uint8_t data){
	uint8_t w_addr = 0;

	w_addr = addr + 0x80;
	if(addr == RTC_MONTH_ADDR){
		check_century();
	}
	if(addr == RTC_HOUR_ADDR){
		if((rtc_hour & AMPM_FLAG) && (data & PM_FLAG)){
			data |= (AMPM_FLAG | PM_FLAG);
		}
		if((rtc_hour & AMPM_FLAG) && !(data & PM_FLAG)){
			data |= AMPM_FLAG;
		}
		if(!(rtc_hour & AMPM_FLAG)){
			data &= ~AMPM_FLAG;
		}
	}
	RTC_CS_PORT &= ~_BV(RTC_CS);
	SPDR = w_addr;
	loop_until_bit_is_set(SPSR, SPIF);
	SPDR = data;
	loop_until_bit_is_set(SPSR, SPIF);
	RTC_CS_PORT |= _BV(RTC_CS);
	RTC_CS_PORT &= ~_BV(RTC_CS);
	RTC_CS_PORT |= _BV(RTC_CS);
}

// raw write rtc
void rtc_raw_write(uint8_t addr, uint8_t data){
	uint8_t w_addr = 0;

	w_addr = addr + 0x80;
	RTC_CS_PORT &= ~_BV(RTC_CS);
	SPDR = w_addr;
	loop_until_bit_is_set(SPSR, SPIF);
	SPDR = data;
	loop_until_bit_is_set(SPSR, SPIF);
	RTC_CS_PORT |= _BV(RTC_CS);
	RTC_CS_PORT &= ~_BV(RTC_CS);
	RTC_CS_PORT |= _BV(RTC_CS);
}

// clear rtc time
void rtc_clear(void){
	rtc_raw_write(RTC_ALARM2_DATE_ADDR, 1);
	rtc_raw_write(RTC_ALARM2_HOUR_ADDR, 0);
	rtc_raw_write(RTC_ALARM2_MIN_ADDR,  0);
	rtc_raw_write(RTC_ALARM1_DATE_ADDR, 1);
	rtc_raw_write(RTC_ALARM1_HOUR_ADDR, 0);
	rtc_raw_write(RTC_ALARM1_MIN_ADDR,  0);
	rtc_raw_write(RTC_ALARM1_SEC_ADDR,  0);
	rtc_raw_write(RTC_YEAR_ADDR,        0);
	rtc_raw_write(RTC_MONTH_ADDR,       1);
	rtc_raw_write(RTC_DATE_ADDR,        1);
	rtc_raw_write(RTC_DAY_ADDR,         1);
	rtc_raw_write(RTC_HOUR_ADDR,        0);
	rtc_raw_write(RTC_MIN_ADDR,         0);
	rtc_raw_write(RTC_SEC_ADDR,         0);
	rtc_raw_write(RTC_CONTROL_ADDR, 0b00000000); // 1Hz, INT output
	rtc_raw_write(RTC_CONTROL_STATUS_ADDR, (RTC_STATUS_OSF | RTC_STATUS_BB32 | RTC_STATUS_EN32));
}
