#ifndef __RTC_H
#define __RTC_H

#define RTC_CS_PORT PORTB
#define RTC_CS PB0

void init_rtc(void);
uint8_t rtc_read(uint8_t addr);
void rtc_write(uint8_t addr, uint8_t data);
//void check_century(void);
void rtc_raw_write(uint8_t addr, uint8_t data);
void rtc_clear(void);

// rtc read addresses
// add 0x80 for write address
// in rtc_write, add 0x80 automatically
#define RTC_SEC_ADDR            0x00
#define RTC_MIN_ADDR            0x01
#define RTC_HOUR_ADDR           0x02
#define RTC_DAY_ADDR            0x03
#define RTC_DATE_ADDR           0x04
#define RTC_MONTH_ADDR          0x05
#define RTC_YEAR_ADDR           0x06
#define RTC_ALARM1_SEC_ADDR     0x07
#define RTC_ALARM1_MIN_ADDR     0x08
#define RTC_ALARM1_HOUR_ADDR    0x09
#define RTC_ALARM1_DATE_ADDR    0x0A
#define RTC_ALARM2_MIN_ADDR     0x0B
#define RTC_ALARM2_HOUR_ADDR    0x0C
#define RTC_ALARM2_DATE_ADDR    0x0D
#define RTC_CONTROL_ADDR        0x0E
#define RTC_CONTROL_STATUS_ADDR 0x0F
#define RTC_AGING_OFFSET_ADDR   0x10
#define RTC_TEMP_MSB_ADDR       0x11 // raed only
#define RTC_TEMP_LSB_ADDR       0x12 // read only
#define RTC_DIS_TEMP_CONV_ADDR  0x13
#define RTC_SRAM_ADDR           0x18
#define RTC_SRAM_DATA_ADDR      0x19

// default set is 2000/01/01 (Sat)
#define RTC_SEC_DEFAULT         0
#define RTC_MIN_DEFAULT         0
#define RTC_HOUR_DEFAULT        0
#define RTC_DAY_DEFAULT         7
#define RTC_DATE_DEFAULT        1
#define RTC_MONTH_DEFAULT       1
#define RTC_YEAR_DEFAULT        0
#define RTC_ALARM1_SEC_DEFAULT  0
#define RTC_ALARM1_MIN_DEFAULT  0
#define RTC_ALARM1_HOUR_DEFAULT 0
#define RTC_ALARM1_DATE_DEFAULT 1
#define RTC_ALARM2_MIN_DEFAULT  0
#define RTC_ALARM2_HOUR_DEFAULT 0
#define RTC_ALARM2_DATE_DEFAULT 1

#define RTC_CONTROL_EOSC  0b10000000
#define RTC_CONTROL_BBSQW 0b01000000
#define RTC_CONTROL_CONV  0b00100000
#define RTC_CONTROL_RS2   0b00010000
#define RTC_CONTROL_RS1   0b00001000
#define RTC_CONTROL_INTCN 0b00000100
#define RTC_CONTROL_A2IE  0b00000010
#define RTC_CONTROL_A1IE  0b00000001

#define RTC_STATUS_OSF    0b10000000
#define RTC_STATUS_BB32   0b01000000
#define RTC_STATUS_CRATE1 0b00100000
#define RTC_STATUS_CRATE2 0b00010000
#define RTC_STATUS_EN32   0b00001000
#define RTC_STATUS_BSY    0b00000100
#define RTC_STATUS_A2F    0b00000010
#define RTC_STATUS_A1F    0b00000001

#define CENTURY_FLAG 0b10000000
#define AMPM_FLAG    0b01000000
#define PM_FLAG      0b00100000

#define TEMP_UPDATE 5

#endif
